'''
Created on Oct 9, 2012

@author: larsab
'''

import Tkinter

# Holder for pcl data
class PclScreen:
    def __init__(self):
        # Deaults
        self.rowHeight = 20
        self.canvasWidth = 500
        self.labelWidth = 50
        self.windowHeight = 525
        self.windowWidth = 525
        
        self.markedGenes = []

    def loadPclFile(self, filename, logger=None):
        fp = open(filename)
        
        # parse header line
        line = fp.readline()
        if line == '':
            if logger != None:
                logger.warn('Invalid file, no header line: %s' % (filename))
            return None
        line.strip()
        sampleIds = line.split('\t')[3:] # first 3 columns are gene id, gene name, and gweight
        
        # Ignore eweight line
        line = fp.readline()
        if line == '':
            if logger != None:
                logger.warn('Invalid file, no header line: %s' % (filename))
        
        # Parse expression values
        geneIds = []   # same order as in file
        expValues = [] # same order as geneIds
        while True:
            line = fp.readline()
            if line == '':
                break
            
            line = line.strip()
            cols = line.split('\t')
            geneIds.append(cols[0])
            exps = []
            for c in cols[3:]:
                exps.append(float(c))
            expValues.append(exps)
        
        fp.close()    
        return (geneIds, sampleIds, expValues)
    
    def loadMappingFile(self, filename, logger=None):
        fp = open(filename)
        
        geneId2Name = {} # Gene ID to name mappings
        
        while True:
            line = fp.readline()
            if line == '':
                break
            
            line = line.strip()
            name, gid = line.split('\t')
            if gid in geneId2Name:
                # Check if the new mappnig is 'better'
                oldName = geneId2Name[gid]
                if (oldName.find('_at') != -1 or oldName.isdigit()) and (name.find('_at') == -1 and not name.isdigit()):
                    geneId2Name[gid] = name
            else:
                geneId2Name[gid] = name
        
        fp.close()
        return geneId2Name
    
    
    def convertTranscriptToHexColor(self, val, cutoff):
        color = '#'            
                
        if val >= 0:    
            if val < cutoff:
                amount = int(255 * (val / cutoff))
                if amount < 16:
                    color += "0";                
                color += hex(amount)[2:]
                
            else:
                color += "FF"
                 
            color += "00"            
            color += "00"
        else:
            color += "AAAAAA"
           
        return color
    
    def convertFoldToHexColor(self, val, cutoff):
        color = "#"    
        if val > 0:
            # Red
            if val < cutoff:
                amount = int(255 * (val / cutoff))
                if amount < 16:
                    color += "0";            
                color += hex(amount)[2:]        
            else:
                color += "FF"
            # Green
            color += "00"
            # Blue        
            color += "00";
        else:     
            # Red   
            color += "00"
            # Green
            if val > (-1 * cutoff):
                amount = int(255 * ((-1 * val) / cutoff))
                if amount < 16:
                    color += "0"            
                color += hex(amount)[2:]        
            else:
                color += "FF";
            # Blue
            color += "00";            
        
        return color
            
    def createHeatmap(self, root, geneIds, sampleIds, expValues, geneId2Name, rowHeight=20, canvasWidth=500, labelWidth=50):
         
        canvasHeight = (len(geneIds) + 1) * rowHeight
        self.canvasHeight = canvasHeight
        columnWidth = (canvasWidth - labelWidth)  / len(sampleIds)
        self.columnWidth = columnWidth
        
        frame = Tkinter.Frame(root, width=columnWidth, height=canvasHeight)
        frame.place(x=0, y=0)
        frame.pack(fill=Tkinter.BOTH, expand=1)    
        canvas = Tkinter.Canvas(frame, width=columnWidth, height=canvasHeight, scrollregion=(0,0,columnWidth,canvasHeight))
        vbar = Tkinter.Scrollbar(frame,orient=Tkinter.VERTICAL)
        vbar.pack(side=Tkinter.RIGHT,fill=Tkinter.Y)
        vbar.config(command=canvas.yview)
        canvas.place(x=0, y=0)
        canvas.config(yscrollcommand=vbar.set)
        canvas.pack(fill=Tkinter.BOTH, expand=True)
        
        # Paint gene IDs
        rowIDs = []
        for i in range(len(geneIds)):
            name = geneIds[i]
            if name in geneId2Name:
                name = geneId2Name[name]
            canvas.create_text(0, (i * rowHeight) + rowHeight, text=name, anchor=Tkinter.NW)
            rowIDs.append(name)
            
        # Paint column IDs
        for i in range(len(sampleIds)):
            canvas.create_text((i * columnWidth) + labelWidth, 0, text=str(i), anchor=Tkinter.NW) 
            
        cutoff = 2.0
        centered = True
        # Paint expression values
        for i in range(len(geneIds)):
            avg = 0.0
            if centered:            
                count = 0.0
                for e in expValues[i]:
                    avg = avg + e
                    count = count + 1
                if count > 0:
                    avg = avg / count
            
            for j in range(len(sampleIds)):
                x = (j * columnWidth) + labelWidth
                y = (i * rowHeight) + rowHeight
                val = expValues[i][j]
                if centered:
                    color = self.convertFoldToHexColor(val - avg, cutoff)            
                else:
                    color = self.convertFoldToHexColor(val, cutoff)
                    
                canvas.create_rectangle(x, y, x + columnWidth, y + rowHeight, fill=color)
                
        return (canvas, rowIDs)       
  
    def markGene(self, canvas, rowIDs, markID, rowHeight=20, canvasWidth=500):
        if markID not in rowIDs:
            return
        
        rowIndex = rowIDs.index(markID)
        y = rowHeight * (rowIndex + 1)
        canvas.create_rectangle(0, y, canvasWidth, y + rowHeight, outline='yellow')
        self.markedGenes.append(markID)
        
    def showOnlyMarkedGenes(self, canvas, geneIds, sampleIds, expValues, geneId2Name, rowHeight=20, canvasWidth=500, labelWidth=50):
        canvas.delete(Tkinter.ALL)
        
        columnWidth = self.columnWidth
        
        # Paint gene IDs
        indexesToDraw = []
        y = rowHeight
        for i in range(len(geneIds)):
            name = geneIds[i]
            if name in geneId2Name:
                name = geneId2Name[name]
            if name in self.markedGenes:
                canvas.create_text(0, y, text=name, anchor=Tkinter.NW)
                indexesToDraw.append(i)
                y = y + rowHeight
            
        # Paint column IDs
        for i in range(len(sampleIds)):
            canvas.create_text((i * columnWidth) + labelWidth, 0, text=str(i), anchor=Tkinter.NW) 
            
        cutoff = 2.0
        centered = True
        # Paint expression values
        y = rowHeight
        for i in indexesToDraw:
            avg = 0.0
            if centered:            
                count = 0.0
                for e in expValues[i]:
                    avg = avg + e
                    count = count + 1
                if count > 0:
                    avg = avg / count
            
            for j in range(len(sampleIds)):
                x = (j * columnWidth) + labelWidth                
                val = expValues[i][j]
                if centered:
                    color = self.convertFoldToHexColor(val - avg, cutoff)            
                else:
                    color = self.convertFoldToHexColor(val, cutoff)
                    
                canvas.create_rectangle(x, y, x + columnWidth, y + rowHeight, fill=color)
            y = y + rowHeight
        

def loadData(stateObject, logger, pclFilename):
    geneIds, sampleIds, expValues = stateObject.loadPclFile(pclFilename, logger)
    stateObject.geneIds = geneIds
    stateObject.sampleIds = sampleIds
    stateObject.expValues = expValues
    
def vizHeatmap(stateObject, logger, args):
    stateObject.root.geometry("%dx%d" % (stateObject.windowWidth, stateObject.windowHeight))
    c, r = stateObject.createHeatmap(stateObject.root, stateObject.geneIds, stateObject.sampleIds, stateObject.expValues, 
                                     stateObject.geneId2Name, stateObject.rowHeight, stateObject.canvasWidth, stateObject.labelWidth)
    stateObject.canvas = c
    stateObject.rowIDs = r
    stateObject.canvas.update()
    
def vizMarkGene(stateObject, logger, geneName):
    stateObject.markGene(stateObject.canvas, stateObject.rowIDs, geneName, stateObject.rowHeight, stateObject.canvasWidth)
    stateObject.canvas.update()
    
def filterMarked(stateObject, logger, minCnt):
    if len(stateObject.markedGenes) >= minCnt:
        return True
    else:
        return False
    
def vizOnlyMarked(stateObject, logger, geneName):
    stateObject.showOnlyMarkedGenes(stateObject.canvas, stateObject.geneIds, stateObject.sampleIds, stateObject.expValues, 
                                    stateObject.geneId2Name, stateObject.rowHeight, stateObject.canvasWidth, stateObject.labelWidth)
   
def getPclFiles(pclDir):    
    import os
    files = os.listdir(pclDir)
    # The list of PCl files is used as arguments
    pclFiles = []
    for f in files:
        if f.find('.pcl') != -1:
            pclFiles.append(os.path.join(pclDir, f))
    pclFiles.sort()
    return pclFiles
    
def runHidra(pclDir, mapFile, nWindows, nProcs):
    # Initialize common variables
    stateObject = PclScreen()
    stateObject.geneId2Name = stateObject.loadMappingFile(mapFile)
    
    # Find pcl files
    pclFiles = getPclFiles(pclDir)
    nTasks = len(pclFiles)

    # Start coordinator
    from bealjas.coordinator.windowCoordinator import WindowCoordinator
    from bealjas.utils.bealjas2tkinter import createWindow, guiLoop, closeWindow, setWindowPosSize, getWindowPosSize, setWindowVisible            
    coordinator = WindowCoordinator(nWindows, nProcs, createWindow, guiLoop, closeWindow, setWindowVisible, getWindowPosSize, setWindowPosSize, stateObject)
    
    # Load task specific dataset
    coordinator.startSlaves(loadData, pclFiles)
    # and draw heatmap
    coordinator.visualize(vizHeatmap, [None] * nTasks)
    raw_input("Enter to mark genes")   
    # Then mark a couple of genes
    coordinator.visualize(vizMarkGene, ['PRKCB'] * nTasks)
    coordinator.visualize(vizMarkGene, ['TNF'] * nTasks)
    coordinator.visualize(vizMarkGene, ['CCR3'] * nTasks)
    coordinator.visualize(vizMarkGene, ['TRD'] * nTasks)
    raw_input("Enter to filter on marked: ")
    coordinator.pendingFilterVisible(filterMarked, [3] * nTasks)
    raw_input("Enter to show only marked: ")
    coordinator.visualize(vizOnlyMarked, [None] * nTasks)
    
    # And let user switch between windows
    coordinator.consoleDriver()
    coordinator.killSlaves()
            
            
def runDebug():
    import sys    
    
    assert(len(sys.argv) == 3), 'Usage: python %s pclFilename mappingFilename' % (sys.argv[0])
    pclFilename = sys.argv[1]
    mappingFilename = sys.argv[2]
    
    stateObject = PclScreen()
    geneIds, sampleIds, expValues = stateObject.loadPclFile(pclFilename)
    geneId2Name = stateObject.loadMappingFile(mappingFilename)
    root = Tkinter.Tk()
    root.geometry("525x525+100+100")
    canvas, rowIDs = stateObject.createHeatmap(root, geneIds, sampleIds, expValues, geneId2Name)
    markGene(canvas, rowIDs, 'PRKCB')
    root.mainloop()
    
if __name__ == '__main__':
    import sys    
    
    assert(len(sys.argv) == 5), 'Usage: python %s pclDir mappingFilename nProcs nWindows' % (sys.argv[0])
    runHidra(sys.argv[1], sys.argv[2], int(sys.argv[3]), int(sys.argv[4]))
    