'''
Function for setting up logging
'''

import logging, os

from bealjas.plotter.plotterError import PlotterError

def setupConsoleLogger(pid, level, optionalArguments=None):
    '''
    Setup logging to console

    @param pid: process identifier used in the logger output
    @param level: logging level to use. Must be one of logging.[DEBUG, INFO, WARNING, ERROR, CRITICAL]
    @param optionalArguments: ignored
    @return: (logger object, None)
    '''
    logger = logging.getLogger(pid)
    logger.setLevel(level)
    ch = logging.StreamHandler()
    ch.setLevel(level)
    logger.addHandler(ch)
    
    print "Console logger started with log level: %s" % (logging.getLevelName(level))
    
    return (logger, None)
    

def setupFileLogger(pid, level, optionalArguments=None):
    '''
    Setup logging to a file
    
    @param pid: process identifier used in the logger output and output filename
    @param level: logging level to use. Must be one of logging.[DEBUG, INFO, WARNING, ERROR, CRITICAL]
    @param optionalArguments: directory where the key 'dir' specifies the logfile directory.
    @return (logger object, log filename).
    '''
    
    if not optionalArguments.has_key('dir'):
        raise PlotterError('Log directory is not specified')

    logDir = optionalArguments['dir']
    if not os.path.isdir(logDir):
        os.makedirs(logDir)
        if not os.path.isdir(logDir):
            raise PlotterError('Log directory %s does not exist and could not be created' % (logDir))            
    
    logfile = os.path.join(logDir, pid)
    
    logger = logging.getLogger(pid)
    logger.setLevel(level)
    fh = logging.FileHandler(logfile)
    fh.setLevel(level)
    logger.addHandler(fh)
    
    print "File logger started with log level: %s and log file: %s\n" % (logging.getLevelName(level), logfile)
    
    return (logger, logfile)
