import unittest

from bealjas.coordinator.bealjasCoordinator import BealjasCoordinator
from bealjas.coordinator.coordinatorError import CoordinatorError
import logging, tempfile, types
from bealjas.utils.setupLogging import setupFileLogger, setupConsoleLogger
     
class BealjasCoordinatorTest(unittest.TestCase):


    def setUp(self):
        self.stateObject = {'screen': None}
        self.coordinator = BealjasCoordinator(self.stateObject)
        self.tmpLogDir = tempfile.mkdtemp()

    def tearDown(self):
        pass

    def testConstructor(self):
        self.assertEqual(self.stateObject, self.coordinator.stateObject)
        self.assertTrue(isinstance(self.coordinator.logger, logging.Logger))
        self.assertIsNone(self.coordinator.logFilename)
        
        # default slave logger setup test
        self.assertTrue(isinstance(self.coordinator.slaveLoggerSetupFunction, types.FunctionType))
        self.assertEquals(setupConsoleLogger, self.coordinator.slaveLoggerSetupFunction)
        self.assertEquals(logging.INFO, self.coordinator.slaveLoggerLevel)
        self.assertIsNone(self.coordinator.slaveLoggerOptionalArguments)
        
    def testSlaveLoggerSetup(self):
        self.coordinator.slaveLoggerSetup(setupFileLogger, logging.WARN, {'dir':self.tmpLogDir})
        
        self.assertTrue(isinstance(self.coordinator.slaveLoggerSetupFunction, types.FunctionType))
        self.assertEquals(setupFileLogger, self.coordinator.slaveLoggerSetupFunction)
        self.assertEquals(logging.WARN, self.coordinator.slaveLoggerLevel)
        self.assertTrue(self.coordinator.slaveLoggerOptionalArguments.has_key('dir'))
        
    # Invalid function
    def testSlaveLoggerSetup2(self):
        self.assertRaises(CoordinatorError, self.coordinator.slaveLoggerSetup, ('foo', 'bar'), logging.WARN, {'dir':self.tmpLogDir})

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()