import unittest

from bealjas.coordinator.bealjasCoordinator import BealjasTask
from bealjas.coordinator.coordinatorError import CoordinatorError

def foo():
    return False
     
class BealjasTaskTest(unittest.TestCase):


    def setUp(self):
        self.task = BealjasTask(3)

    def tearDown(self):
        pass

    def testConstructor(self):
        self.assertEqual(3, self.task.taskID)
        self.assertEquals(0, len(self.task.taskArguments))
        self.assertIsNone(self.task.slave)
        self.assertIsNone(self.task.pipe)
        self.assertFalse(self.task.isVisible)
        self.assertIsNone(self.task.visibleFilter)
        self.assertIsNone(self.task.pendingVisibleFilter)
        self.assertFalse(self.task.isClosed)
        self.assertIsNone(self.task.showTimestamp)
        
    def testGetNFunctionCalls(self):
        self.assertEquals(0, self.task.getNFunctionCalls())
        self.task.taskArguments.append(1)
        self.task.taskArguments.append(1)
        self.task.taskArguments.append(1)        
        self.assertEquals(3, self.task.getNFunctionCalls())
        
    def testSetRemoveSlave(self):
        # The input types are ignore
        self.task.setSlave(1, 2)
        self.assertEqual(1, self.task.slave)
        self.assertEqual(2, self.task.pipe)
        self.task.removeSlave()
        self.assertIsNone(self.task.slave)
        self.assertIsNone(self.task.pipe)
    
    def testIsRunning(self):
        self.assertFalse(self.task.isRunning())
        # The input types are ignore
        self.task.setSlave(1, 2)
        self.assertTrue(self.task.isRunning())
        
    def testIsShowable(self):
        # Initialzied to hidden        
        self.assertTrue(self.task.isShowable())
        
        # Check all possible permutations
        self.task.isVisible = False
        self.task.visibleFilter = False
        self.task.isClosed = False
        self.assertFalse(self.task.isShowable())
        
        self.task.isVisible = False
        self.task.visibleFilter = False
        self.task.isClosed = True
        self.assertFalse(self.task.isShowable())
        
        self.task.isVisible = False
        self.task.visibleFilter = True
        self.task.isClosed = False
        self.assertTrue(self.task.isShowable())
        
        self.task.isVisible = False
        self.task.visibleFilter = True
        self.task.isClosed = True
        self.assertFalse(self.task.isShowable())
        
        self.task.isVisible = True
        self.task.visibleFilter = False
        self.task.isClosed = False
        self.assertRaises(CoordinatorError, self.task.isShowable)
        
        self.task.isVisible = True
        self.task.visibleFilter = False
        self.task.isClosed = True
        self.assertRaises(CoordinatorError, self.task.isShowable)
        
        self.task.isVisible = True
        self.task.visibleFilter = True
        self.task.isClosed = False
        self.assertFalse(self.task.isShowable())
        
        self.task.isVisible = True
        self.task.visibleFilter = True
        self.task.isClosed = True
        self.assertRaises(CoordinatorError, self.task.isShowable)
        
        self.task.isVisible = False
        self.task.visibleFilter = None
        self.task.isClosed = False
        self.assertTrue(self.task.isShowable())
        
        self.task.isVisible = False
        self.task.visibleFilter = None
        self.task.isClosed = True
        self.assertFalse(self.task.isShowable())
        
        self.task.isVisible = True
        self.task.visibleFilter = None
        self.task.isClosed = False
        self.assertFalse(self.task.isShowable())
        
        self.task.isVisible = True
        self.task.visibleFilter = None
        self.task.isClosed = True
        self.assertRaises(CoordinatorError, self.task.isShowable)
        
    
if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()