'''
Created on Jun 22, 2012

@author: larsab
'''
import unittest

import logging
import os
import shutil
import tempfile

from multiprocessing import Pipe

from bealjas.plotter.bealjasPlotter import BealjasPlotter
from bealjas.utils.setupLogging import setupConsoleLogger, setupFileLogger

class Test(unittest.TestCase):

    def setUp(self):
        self.tmpLogDir = tempfile.mkdtemp()
        self.stateObject = [1, 2, 3]
        
    def tearDown(self):
        #shutil.rmtree(self.tmpLogDir)
        pass

    def testConstructor(self):
        parentEnd, childEnd = Pipe()
                
        bealjasPlotter = BealjasPlotter("unit-plotter", childEnd, setupFileLogger, logging.INFO, {'dir': self.tmpLogDir}, stateObject=self.stateObject)
        self.assertTrue("unit-plotter", bealjasPlotter.name)
        self.assertTrue(childEnd, bealjasPlotter.pipe)
        self.assertTrue(self.stateObject, bealjasPlotter.stateObject)
        self.assertTrue(isinstance(bealjasPlotter.logger, logging.Logger))
        self.assertTrue(bealjasPlotter.logFilename.startswith(self.tmpLogDir))
        self.assertTrue(os.path.isfile(bealjasPlotter.logFilename))
        
    def testConstructor2(self):
        parentEnd, childEnd = Pipe()
                
        bealjasPlotter = BealjasPlotter("unit-plotter", childEnd, setupConsoleLogger, logging.INFO, {'dir': self.tmpLogDir}, stateObject=self.stateObject)
        self.assertTrue("unit-plotter", bealjasPlotter.name)
        self.assertTrue(childEnd, bealjasPlotter.pipe)
        self.assertTrue(self.stateObject, bealjasPlotter.stateObject)
        self.assertTrue(isinstance(bealjasPlotter.logger, logging.Logger))
        self.assertIsNone(bealjasPlotter.logFilename)
        
    def testReadLogfileLines(self):
        parentEnd, childEnd = Pipe()                
        bealjasPlotter = BealjasPlotter("unit-plotter", childEnd, setupFileLogger, logging.INFO, {'dir': self.tmpLogDir}, stateObject=self.stateObject)
        
        bealjasPlotter.logger.debug('debug')
        bealjasPlotter.logger.info('info')
        bealjasPlotter.logger.warn('warn')
        bealjasPlotter.logger.error('error')
        bealjasPlotter.logger.critical('critical')
    
        logLines = bealjasPlotter.readLogfileLines()
        self.assertEquals(4, len(logLines))
        self.assertEquals('info\n', logLines[0])
        self.assertEquals('warn\n', logLines[1])
        self.assertEquals('error\n', logLines[2])
        self.assertEquals('critical\n', logLines[3])
        
    # Invalid logging function
    def testReadLogfileLines2(self):
        parentEnd, childEnd = Pipe()                
        bealjasPlotter = BealjasPlotter("unit-plotter", childEnd, setupConsoleLogger, logging.INFO, {'dir': self.tmpLogDir}, stateObject=self.stateObject)
        
        bealjasPlotter.logger.debug('debug')
        bealjasPlotter.logger.info('info')
        bealjasPlotter.logger.warn('warn')
        bealjasPlotter.logger.error('error')
        bealjasPlotter.logger.critical('critical')
    
        logLines = bealjasPlotter.readLogfileLines()
        self.assertIsNone(logLines)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()